﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Configuration;
using System.IO.Ports;
using HttpServer;
using System.Threading;

namespace Roomby
{
    public partial class MainWindow : Form
    {

        public MainWindow()
        {
            InitializeComponent();

            JeeNodeStart(); 
            StartWebServer();
        }

        #region SerialPort

        SerialPort JeeNodePort = null;
        private static object commLock = new object();

        public void JeeNodeStart()
        {
            string port = Convert.ToString(ConfigurationSettings.AppSettings["JeeNodePort"]);
            int baud = Convert.ToInt32(ConfigurationSettings.AppSettings["JeeNodeBaudRate"]);

            try
            {
                SetJeeNodeStatus("Starting...");
                JeeNodePort = new SerialPort(port, baud, Parity.None, 8, StopBits.One);
                JeeNodePort.Handshake = Handshake.None;
                JeeNodePort.NewLine = "\r\n";
                JeeNodePort.Open();
                SetJeeNodeStatus("Ready");
            }
            catch (Exception ex)
            {
                SetJeeNodeStatus("Error");
            }
            finally
            {
                UpdateGUI();
            }
        }

        public void JeeNodeStop()
        {
            try
            {
                SetJeeNodeStatus("Stopping...");
                if (JeeNodePort != null)
                {
                    JeeNodePort.Close();
                    JeeNodePort = null;
                }
                SetJeeNodeStatus("Stopped");
            }
            catch (Exception ex)
            {
                SetJeeNodeStatus("Error");
            }
            finally
            {
                UpdateGUI();
            }
        }

        private String JeeNodeSend(String sendData)
        {
            if (!JeeNodePort.IsOpen) JeeNodePort.Open();

            String receiveData;
            lock (commLock)
            {
                SetJeeNodeStatus("Sending...");
                string existingData = JeeNodePort.ReadExisting();
                JeeNodePort.Write(sendData);
                receiveData = JeeNodePort.ReadLine();
                SetJeeNodeStatus("Ready");
            }
            return receiveData;
        }

        #endregion

        #region WebServer

        HttpServer.HttpListener WebServer = null;

        public void StartWebServer()
        {
            try
            {
                SetHttpNodeStatus("Starting...");
                WebServer = HttpServer.HttpListener.Create(IPAddress.Any, Convert.ToInt16(ConfigurationSettings.AppSettings["HttpPort"]));
                WebServer.RequestReceived += new EventHandler<HttpServer.RequestEventArgs>(WebServer_RequestReceived);
                WebServer.Start(5);
                SetHttpNodeStatus("Ready"); 
            }
            catch (Exception ex)
            {
                SetHttpNodeStatus("Error");
            }
            finally
            {
                UpdateGUI();
            }
        }

        private void WebServer_RequestReceived(object sender, HttpServer.RequestEventArgs e)
        {
            IHttpClientContext context = (IHttpClientContext)sender;
            IHttpRequest request = e.Request;
            
            SetHttpNodeStatus("Receiving"); 
            
            if (request.UriParts.Length == 2)
            {
                String authorization = request.UriParts[1];
                String command = request.UriParts[0].Replace("%20", "+");
                String remoteIP = request.RemoteEndPoint.Address.ToString();

                if (authorization == Convert.ToString(ConfigurationSettings.AppSettings["HttpAuthorization"]))
                {
                    if (command.Length == 3)
                    {
                        Log("HTTP [" + remoteIP + "] <- " + command);
                        bool resp = Command(command);
                        string responce = resp ? "OK" : "FAILED";
                        context.Respond(responce);
                    }
                }
                else
                {
                    Log("Unauthorized access from " + remoteIP);
                    //context.Respond("Not authorized");
                }
            }
            SetHttpNodeStatus("Ready"); 
        }

        #endregion

        #region Helpers

        private void UpdateGUI()
        {
            groupBox3.Enabled = groupBox2.Enabled = (labJeeNodeStatus.Text == "Ready" && labJeeNodeStatus.Text == "Ready");
        }

        #endregion

        #region Commander logic

        delegate void SetJeeNodeStatusCallback(string text);
        private void SetJeeNodeStatus(string text)
        {
            if (this.labJeeNodeStatus.InvokeRequired)
            {
                SetJeeNodeStatusCallback d = new SetJeeNodeStatusCallback(SetJeeNodeStatus);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.labJeeNodeStatus.Text = text;
            }
        }

        delegate void SetHttpStatusCallback(string text);
        private void SetHttpNodeStatus(string text)
        {
            if (this.labHttpStatus.InvokeRequired)
            {
                SetHttpStatusCallback d = new SetHttpStatusCallback(SetHttpNodeStatus);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.labHttpStatus.Text = text;
            }
        }

        delegate void LogCallback(string text);
        private void Log(string text)
        {
            if (this.lB.InvokeRequired)
            {
                LogCallback d = new LogCallback(Log);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.lB.Items.Add(String.Concat(DateTime.Now.ToShortDateString(), " ", DateTime.Now.ToLongTimeString(), " - ", text));
                this.lB.TopIndex = this.lB.Items.Count - 1;
            }
        }

        private bool Command(String command)
        {
            if (!bW.IsBusy)
            {
                bW.RunWorkerAsync(new DoWorkEventArgs(command));
                Log("Roomba <- " + command);
                return true;
            }
            else
            {
                Log("Roomba *");
                return false;
            }
        }

        private void bW_DoWork(object sender, DoWorkEventArgs e)
        {
            DoWorkEventArgs a = (DoWorkEventArgs)e.Argument;
            String command = a.Argument as String;
            UpdateRoombaIndicator(null);
            string response = JeeNodeSend(command);
            e.Result = response;
        }

        private void bW_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            String resultStr = e.Result as String;
            Log("Roomba -> " + resultStr);
            bool resultBool = resultStr.Equals("OK");
            UpdateRoombaIndicator(resultBool);
        }

        delegate void roombaCommandDelegate(bool? responce);
        void UpdateRoombaIndicator(bool? responce)
        {
            if (this.InvokeRequired)
            {
                roombaCommandDelegate del = new roombaCommandDelegate(UpdateRoombaIndicator);
                this.Invoke(del, new object[] { responce });
            }
            else
            {
                pBGray.Visible = (responce == null); pBGreen.Visible = (responce != null && (bool)responce); pBRed.Visible = (responce != null && !(bool)responce);
            }
        }
        #endregion

        private void button1_Click(object sender, EventArgs e)
        {
            Command("POW");
        }

        private void button2_Click(object sender, EventArgs e)
        {
            Command("STA");
        }

        private void button3_Click(object sender, EventArgs e)
        {
            Command("RST");
        }

        private void button4_Click(object sender, EventArgs e)
        {
            Command("WKP");
        }

        private void button5_Click(object sender, EventArgs e)
        {
            Command("FSD");
        }

        private void button9_Click(object sender, EventArgs e)
        {
            Command("CLN");
        }

        private void button10_Click(object sender, EventArgs e)
        {
            Command("SPT");
        }

        private void button11_Click(object sender, EventArgs e)
        {
            Command("MAX");
        }

        private void button6_Click(object sender, EventArgs e)
        {
            Command("SAF");
        }

        private void button7_Click(object sender, EventArgs e)
        {
            Command("FUL");
        }

        private void button8_Click(object sender, EventArgs e)
        {
            Command("CNT");
        }

        private void button12_Click(object sender, EventArgs e)
        {
            Command("DGF");
        }

        private void button14_Click(object sender, EventArgs e)
        {
            Command("DSL");
        }

        private void button15_Click(object sender, EventArgs e)
        {
            Command("DSR");
        }

        private void button13_Click(object sender, EventArgs e)
        {
            Command("DGB");
        }

        private void button16_Click(object sender, EventArgs e)
        {
            Command("DV+");
        }

        private void button17_Click(object sender, EventArgs e)
        {
            Command("DV-");
        }

        private void button18_Click(object sender, EventArgs e)
        {
            Command("DR+");
        }

        private void button19_Click(object sender, EventArgs e)
        {
            Command("DR-");
        }
    }
}
